"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateCommand = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const utils_1 = require("../../utils");
const Defaults_1 = require("./Defaults");
const errors_1 = require("../../errors");
class CreateCommand {
    _source;
    _destination;
    _options;
    _createCommandResults;
    get source() {
        return this._source;
    }
    set source(source) {
        this._source = source;
    }
    get destination() {
        return this._destination;
    }
    set destination(destination) {
        this._destination = destination;
    }
    get options() {
        return this._options;
    }
    changeOptions(options) {
        if (options)
            this._options = { ...this._options, ...options };
    }
    resetOptions() {
        this._options = Defaults_1.CREATE_COMMAND_DEFAULT_OPTIONS;
    }
    constructor(source, destination, options = Defaults_1.CREATE_COMMAND_DEFAULT_OPTIONS) {
        this._source = source;
        this._destination = destination;
        this._options = { ...Defaults_1.CREATE_COMMAND_DEFAULT_OPTIONS, ...options };
        this._createCommandResults = [];
    }
    _addCreateCommandResult(createCommandResult) {
        this._createCommandResults.push(createCommandResult);
    }
    _clearCreateCommandResults() {
        this._createCommandResults = [];
    }
    _getTemplatesFolderPath() {
        return path_1.default.resolve(process.cwd(), this._options.templatesFolder);
    }
    _getSourcePath(templatesFolderPath) {
        return path_1.default.resolve(templatesFolderPath, this._source);
    }
    _throwTemplatesFolderNotFound(templatesFolderPath) {
        if (!fs_1.default.existsSync(templatesFolderPath)) {
            throw new errors_1.NotFoundError(templatesFolderPath, `The ${templatesFolderPath} folder does not exist`);
        }
    }
    _throwSourcePathNotFound(sourcePath) {
        if (!fs_1.default.existsSync(sourcePath)) {
            throw new errors_1.NotFoundError(sourcePath, `${sourcePath} not found`);
        }
    }
    _getSourceType(sourcePath) {
        const fsStatus = fs_1.default.lstatSync(sourcePath);
        const isDirectory = fsStatus.isDirectory();
        const isFile = fsStatus.isFile();
        return {
            isDirectory,
            isFile,
        };
    }
    _getParsedNamesToReplace() {
        return this._options.replaceNames?.map?.((keyAndName) => {
            const [key, name] = keyAndName.split(this._options.keyValueSeparator);
            return { key, name };
        });
    }
    _throwNamesToReplaceIncorrectlyFormatted(parsedNamesToReplace) {
        if (!parsedNamesToReplace || !this._options.replaceNames)
            return;
        const isReplaceNamesOptionIncorrectly = parsedNamesToReplace.some(({ key, name }) => !key || !name);
        if (isReplaceNamesOptionIncorrectly) {
            const separator = this._options.keyValueSeparator;
            throw new errors_1.SyntaxError({
                message: 'The --replace-names (-rn) option is incorrectly formatted',
                expected: `key1${separator}name1 key2${separator}name2`,
                received: this._options.replaceNames.join(' '),
            });
        }
    }
    _getReplaceContentObject() {
        const replaceContentObject = {};
        this._options.replaceContent?.forEach?.((keyAndText) => {
            const [key, text] = keyAndText.split(this._options.keyValueSeparator);
            replaceContentObject[key] = text;
        });
        return replaceContentObject;
    }
    _throwReplaceContentOptionIncorrectlyFormatted(replaceContentObject) {
        const isReplaceContentOptionIncorrectly = Object.entries(replaceContentObject).some(([key, text]) => !key || !text);
        if (this._options.replaceContent && isReplaceContentOptionIncorrectly) {
            const separator = this._options.keyValueSeparator;
            throw new errors_1.SyntaxError({
                message: 'The --replace-content (-rc) option is incorrectly formatted',
                expected: `key1${separator}text1 key2${separator}text2`,
                received: this._options.replaceContent.join(' '),
            });
        }
    }
    _throwEmptySourceFolder(sourcePath) {
        if (fs_1.default.readdirSync(sourcePath).length === 0) {
            throw new errors_1.NotFoundError(sourcePath, `No files found at ${sourcePath}`);
        }
    }
    _getPathLastPart(path) {
        const pathParts = path.split('/');
        return pathParts[pathParts.length - 1];
    }
    _getReplacedName(name, parsedNamesToReplace) {
        let replacedName = name;
        if (parsedNamesToReplace) {
            parsedNamesToReplace.forEach(({ key, name }) => {
                const keyToUse = this._options.brackets ? `[${key}]` : key;
                if (replacedName.includes(keyToUse)) {
                    replacedName = replacedName.replace(keyToUse, name);
                }
            });
        }
        return replacedName;
    }
    _getSourceName(parsedNamesToReplace) {
        const defaultName = this._getPathLastPart(this._source);
        const replacedName = this._getReplacedName(defaultName, parsedNamesToReplace);
        return this._options.name || replacedName;
    }
    _getDestinationPath(parsedNamesToReplace) {
        const folderName = this._getSourceName(parsedNamesToReplace);
        return path_1.default.resolve(process.cwd(), this._destination, folderName);
    }
    _getFileContent(filePath, replaceContentObject, canReplaceContent) {
        const fileContent = fs_1.default.readFileSync(filePath, 'utf-8');
        const newFileContent = canReplaceContent
            ? utils_1.TextUtils.replaceTextPieces(fileContent, replaceContentObject)
            : fileContent;
        return Buffer.from(newFileContent).toString(this._options.encoding);
    }
    _throwMisusedOptionsError() {
        const hasNameOption = !!this._options.name;
        const hasReplaceNamesOption = !!this._options.replaceNames;
        if (hasNameOption && hasReplaceNamesOption) {
            throw new errors_1.MisusedOptionsError({
                message: 'The --name (-n) option cannot be used together with the --replace-names (-rn) option to prevent unexpected results',
                options: {
                    name: this._options.name,
                    replaceNames: this._options.replaceNames,
                },
            });
        }
    }
    _createFolderRecursively(sourcePath, destinationPath, replaceContentObject, parsedNamesToReplace, canReplaceContent) {
        const folderContentItems = fs_1.default.readdirSync(sourcePath);
        folderContentItems.forEach((itemName) => {
            const itemPath = path_1.default.resolve(sourcePath, itemName);
            const itemStatus = fs_1.default.lstatSync(itemPath);
            const isFolderItemDirectory = itemStatus.isDirectory();
            const isFolderItemFile = itemStatus.isFile();
            const replacedName = this._getReplacedName(itemName, parsedNamesToReplace);
            if (isFolderItemFile) {
                const fileContent = this._getFileContent(itemPath, replaceContentObject, canReplaceContent);
                const fileDestinationPath = path_1.default.resolve(destinationPath, replacedName);
                fs_1.default.writeFileSync(fileDestinationPath, fileContent);
                this._addCreateCommandResult({
                    type: 'file',
                    sourcePath: itemPath,
                    destinationPath: fileDestinationPath,
                });
            }
            else if (isFolderItemDirectory) {
                const nestedFolderPath = path_1.default.resolve(destinationPath, replacedName);
                fs_1.default.mkdirSync(nestedFolderPath);
                this._addCreateCommandResult({
                    type: 'folder',
                    sourcePath: itemPath,
                    destinationPath: nestedFolderPath,
                });
                this._createFolderRecursively(itemPath, nestedFolderPath, replaceContentObject, parsedNamesToReplace, canReplaceContent);
            }
        });
    }
    _throwAlreadyExistsOnDestination(destinationPath) {
        const alreadyExists = fs_1.default.existsSync(destinationPath);
        if (alreadyExists) {
            throw new errors_1.AlreadyExistsError(destinationPath, `Cannot create ${destinationPath} because it already exists`);
        }
    }
    run() {
        this._throwMisusedOptionsError();
        this._clearCreateCommandResults();
        const templatesFolderPath = this._getTemplatesFolderPath();
        this._throwTemplatesFolderNotFound(templatesFolderPath);
        const sourcePath = this._getSourcePath(templatesFolderPath);
        this._throwSourcePathNotFound(sourcePath);
        const parsedNamesToReplace = this._getParsedNamesToReplace();
        this._throwNamesToReplaceIncorrectlyFormatted(parsedNamesToReplace);
        const replaceContentObject = this._getReplaceContentObject();
        this._throwReplaceContentOptionIncorrectlyFormatted(replaceContentObject);
        const canReplaceContent = Object.keys(replaceContentObject).length > 0;
        const destinationPath = this._getDestinationPath(parsedNamesToReplace);
        this._throwAlreadyExistsOnDestination(destinationPath);
        const sourceType = this._getSourceType(sourcePath);
        if (sourceType.isDirectory) {
            this._throwEmptySourceFolder(sourcePath);
            fs_1.default.mkdirSync(destinationPath);
            this._addCreateCommandResult({
                sourcePath,
                destinationPath,
                type: 'folder',
            });
            this._createFolderRecursively(sourcePath, destinationPath, replaceContentObject, parsedNamesToReplace, canReplaceContent);
            return this._createCommandResults;
        }
        else if (sourceType.isFile) {
            const fileContent = this._getFileContent(sourcePath, replaceContentObject, canReplaceContent);
            fs_1.default.writeFileSync(destinationPath, fileContent);
            this._addCreateCommandResult({
                sourcePath,
                destinationPath,
                type: 'file',
            });
            return this._createCommandResults;
        }
        throw new Error('The source can only be a file or a folder');
    }
}
exports.CreateCommand = CreateCommand;
